import time

__version__ = "0.4.1"
__author__ = "Lightning AI et al."
__author_email__ = "pytorch@lightning.ai"
__license__ = "Apache-2.0"
__copyright__ = f"Copyright (c) 2022-{time.strftime('%Y')}, {__author__}."
__homepage__ = "https://github.com/Lightning-AI/utilities"
__docs__ = "PyTorch Lightning Sample project."
__long_doc__ = """
What is it?
-----------
This package allows for sharing GH workflows, CI/CD assistance actions, and Python utilities across the Lightning
ecosystem.
"""

__all__ = [
    "__author__",
    "__author_email__",
    "__copyright__",
    "__docs__",
    "__homepage__",
    "__license__",
    "__version__",
]
